@@ -78,12 +78,14 @@ class PermuteToFrom(nn.Module):
 
 def attention(q, k, v, h):
     b, t, d = q.shape
-    merge_heads = lambda x: x.reshape(b, t, h, -1).transpose(1, 2).reshape(b * h, t, -1)
+    e = d // h
+
+    merge_heads = lambda x: x.reshape(b, -1, h, e).transpose(1, 2).reshape(b * h, -1, e)
     q, k, v = map(merge_heads, (q, k, v))
     dots = torch.einsum('bie,bje->bij', q, k) * ((d // h) ** -0.5)
     dots = dots.softmax(dim=-1)
     out = torch.einsum('bij,bje->bie', dots, v)
-    out = out.reshape(b, h, t, -1).transpose(1, 2).reshape(b, t, -1)
+    out = out.reshape(b, h, -1, e).transpose(1, 2).reshape(b, -1, d)
     return out
 
 class SelfAttention(nn.Module):
